@@ -3,6 +3,7 @@ use std::fmt;
 
 use crate::bits;
 use crate::bits::{LEFT_MASKS, RIGHT_MASKS};
+use crate::constants::PREFIX_TABLE_SIZE_LOG;
 use crate::errors::{QCompressError, QCompressResult};
 use crate::types::UnsignedLike;
 
@@ -53,7 +54,7 @@ impl BitReader {
     }
   }
 
-  pub fn read_bytes(&mut self, n: usize) -> QCompressResult<&[u8]> {
+  pub fn read_aligned_bytes(&mut self, n: usize) -> QCompressResult<&[u8]> {
     self.refresh_if_needed();
 
     if self.j == 0 {
@@ -62,7 +63,11 @@ impl BitReader {
       self.j = 8;
       Ok(res)
     } else {
-      Err(QCompressError::MisalignedError {})
+      Err(QCompressError::invalid_argument(format!(
+        "cannot read aligned bytes on misaligned bit reader at byte {} bit {}",
+        self.i,
+        self.j,
+      )))
     }
   }
 
@@ -83,6 +88,30 @@ impl BitReader {
     res
   }
 
+  // returns (bits read, idx)
+  pub fn read_prefix_table_idx(&mut self) -> (usize, usize) {
+    let n_plus_j = PREFIX_TABLE_SIZE_LOG + self.j;
+    if n_plus_j <= 8 {
+      let shift = 8 - n_plus_j;
+      let res = (self.bytes[self.i] & LEFT_MASKS[self.j] & RIGHT_MASKS[n_plus_j]) >> shift;
+      self.j = n_plus_j;
+      (PREFIX_TABLE_SIZE_LOG, res as usize)
+    } else {
+      let remaining = n_plus_j - 8;
+      let mut res = ((self.bytes[self.i] & LEFT_MASKS[self.j]) as usize) << remaining;
+      self.i += 1;
+      if self.i < self.bytes.len() {
+        let shift = 8 - remaining;
+        res |= ((self.bytes[self.i] & RIGHT_MASKS[remaining]) >> shift) as usize;
+        self.j = remaining;
+        (PREFIX_TABLE_SIZE_LOG, res)
+      } else {
+        self.j = 0;
+        (PREFIX_TABLE_SIZE_LOG - remaining, res)
+      }
+    }
+  }
+
   pub fn read_diff<Diff: UnsignedLike>(&mut self, n: usize) -> Diff {
     if n == 0 {
       return Diff::ZERO;
@@ -91,7 +120,7 @@ impl BitReader {
     self.refresh_if_needed();
 
     let n_plus_j = n + self.j;
-    if n_plus_j < 8 {
+    if n_plus_j <= 8 {
       // it's all in the current byte
       let shift = 8 - n_plus_j;
       let res = Diff::from((self.bytes[self.i] & LEFT_MASKS[self.j] & RIGHT_MASKS[n_plus_j]) >> shift);
@@ -144,20 +173,22 @@ impl BitReader {
     res
   }
 
-  pub fn drain_bytes(&mut self) -> QCompressResult<&[u8]> {
+  pub fn drain_bytes(&mut self) -> &[u8] {
     if self.j != 0 {
       self.i += 1;
       self.j = 0;
     }
     let n = self.bytes.len() - self.i;
-    self.read_bytes(n)
+    self.read_aligned_bytes(n).unwrap() // this cannot fail because we just did byte alignment
   }
 
   // Seek to the end of the byte.
   // Used to skip to the next metadata or body section of the file, since they
   // always start byte-aligned.
   pub fn drain_byte(&mut self) {
-    self.j = 8;
+    if self.j > 0 {
+      self.j = 8;
+    }
   }
 
   pub fn seek_bytes(&mut self, n_bytes: usize) {
@@ -165,10 +196,33 @@ impl BitReader {
     self.i += n_bytes;
   }
 
+  pub fn rewind(&mut self, n: usize) {
+    if n > self.j {
+      self.i -= 1 + (n - self.j - 1).div_euclid(8);
+      self.j = 7 - (n - self.j - 1).rem_euclid(8);
+    } else {
+      self.j -= n;
+    }
+  }
+
   pub fn inds(&self) -> (usize, usize) {
     (self.i, self.j)
   }
 
+  pub fn aligned_byte_ind(&self) -> QCompressResult<usize> {
+    if self.j == 0 {
+      Ok(self.i)
+    } else if self.j == 8 {
+      Ok(self.i + 1)
+    } else {
+      Err(QCompressError::invalid_argument(format!(
+        "cannot get aligned byte index on misaligned bit reader at byte {} bit {}",
+        self.i,
+        self.j,
+      )))
+    }
+  }
+
   pub fn size(&self) -> usize {
     self.bytes.len()
   }
@@ -185,7 +239,7 @@ mod tests {
     let bytes = vec![0x9a, 0x6b, 0x2d];
     let mut bit_reader = BitReader::from(bytes);
     assert_eq!(
-      bit_reader.read_bytes(1)?,
+      bit_reader.read_aligned_bytes(1)?,
       vec![0x9a],
     );
     assert!(!bit_reader.read_one());
@@ -209,4 +263,24 @@ mod tests {
     //leaves 1 bit left over
     Ok(())
   }
+
+  #[test]
+  fn test_rewind() {
+    let mut reader = BitReader {
+      bytes: vec![], // irrelevant
+      i: 5,
+      j: 3,
+    };
+
+    reader.rewind(2);
+    assert_eq!(reader.inds(), (5, 1));
+    reader.rewind(2);
+    assert_eq!(reader.inds(), (4, 7));
+    reader.rewind(7);
+    assert_eq!(reader.inds(), (4, 0));
+    reader.rewind(8);
+    assert_eq!(reader.inds(), (3, 0));
+    reader.rewind(17);
+    assert_eq!(reader.inds(), (0, 7));
+  }
 }
