@@ -4,12 +4,14 @@ pub const MAGIC_TERMINATION_BYTE: u8 = 46; // .
 
 pub const MAX_ENTRIES: u64 = (1_u64 << 24) - 1;
 pub const BITS_TO_ENCODE_N_ENTRIES: u32 = 24;
-pub const MAX_MAX_DEPTH: u32 = 15;
-pub const BITS_TO_ENCODE_PREFIX_LEN: u32 = 4;
+pub const MAX_COMPRESSION_LEVEL: u32 = 12;
 pub const MAX_JUMPSTART: usize = 31;
 pub const BITS_TO_ENCODE_JUMPSTART: u32 = 5;
 pub const BITS_TO_ENCODE_COMPRESSED_BODY_SIZE: u32 = 32;
 
+pub const PREFIX_TABLE_SIZE_LOG: usize = 4; // tuned to maximize performance on intel i5
+pub const PREFIX_TABLE_SIZE: usize = 1_usize << PREFIX_TABLE_SIZE_LOG;
+
 #[cfg(test)]
 mod tests {
   use crate::constants::*;
@@ -24,13 +26,14 @@ mod tests {
     assert_can_encode(BITS_TO_ENCODE_N_ENTRIES, MAX_ENTRIES);
   }
 
-  #[test]
-  fn test_bits_to_encode_prefix_len() {
-    assert_can_encode(BITS_TO_ENCODE_PREFIX_LEN, MAX_MAX_DEPTH as u64);
-  }
-
   #[test]
   fn test_bits_to_encode_jumpstart() {
     assert_can_encode(BITS_TO_ENCODE_JUMPSTART, MAX_JUMPSTART as u64);
   }
+
+  #[test]
+  fn test_prefix_table_size_fits_in_byte() {
+    assert!(PREFIX_TABLE_SIZE_LOG > 0);
+    assert!(PREFIX_TABLE_SIZE_LOG <= 8);
+  }
 }
