@@ -8,18 +8,24 @@ use crate::bit_reader::BitReader;
 use crate::chunk_metadata::{ChunkMetadata, DecompressedChunk};
 use crate::constants::*;
 use crate::errors::{QCompressError, QCompressResult};
-use crate::prefix::{Prefix, PrefixDecompressionInfo};
+use crate::huffman_decoding::HuffmanTable;
+use crate::prefix::Prefix;
 use crate::types::{NumberLike, UnsignedLike};
 use crate::utils;
 
 #[derive(Clone, Debug, Default)]
 pub struct DecompressorConfig {}
 
-fn validate_prefix_tree<T: NumberLike>(prefixes: &[Prefix<T>], max_depth: u32) -> QCompressResult<()> {
+fn validate_prefix_tree<T: NumberLike>(prefixes: &[Prefix<T>]) -> QCompressResult<()> {
   if prefixes.is_empty() {
     return Ok(());
   }
 
+  let mut max_depth = 0;
+  for p in prefixes {
+    max_depth = max(max_depth, p.val.len() as u32);
+  }
+
   let max_n_leafs = 1_usize << max_depth;
   let mut is_specifieds = vec![false; max_n_leafs];
   for p in prefixes {
@@ -27,15 +33,21 @@ fn validate_prefix_tree<T: NumberLike>(prefixes: &[Prefix<T>], max_depth: u32) -
     let n_leafs = 1_usize << (max_depth - p.val.len() as u32);
     for is_specified in is_specifieds.iter_mut().skip(base_idx).take(n_leafs) {
       if *is_specified {
-        return Err(QCompressError::HuffmanTreeError {bits: p.val.clone(), issue: "duplicated"});
+        return Err(QCompressError::corruption(format!(
+          "multiple prefixes for {} found in chunk metadata",
+          bits::bits_to_string(&p.val),
+        )));
       }
       *is_specified = true;
     }
   }
   for (idx, is_specified) in is_specifieds.iter().enumerate() {
     if !is_specified {
-      let bits = bits::usize_truncated_to_bits(idx, max_depth);
-      return Err(QCompressError::HuffmanTreeError {bits, issue: "missing"});
+      let val = bits::usize_truncated_to_bits(idx, max_depth);
+      return Err(QCompressError::corruption(format!(
+        "no prefixes for {} found in chunk metadata",
+        bits::bits_to_string(&val),
+      )));
     }
   }
   Ok(())
@@ -43,12 +55,9 @@ fn validate_prefix_tree<T: NumberLike>(prefixes: &[Prefix<T>], max_depth: u32) -
 
 #[derive(Clone)]
 struct ChunkDecompressor<T> where T: NumberLike {
+  huffman_table: HuffmanTable<T::Unsigned>,
   prefixes: Vec<Prefix<T>>,
-  prefix_map: Vec<PrefixDecompressionInfo<T::Unsigned>>,
-  prefix_len_map: Vec<u32>,
-  max_depth: u32,
   n: usize,
-  is_single_prefix: bool,
   compressed_body_size: usize,
 }
 
@@ -64,61 +73,28 @@ impl<T> ChunkDecompressor<T> where T: NumberLike {
       ..
     } = metadata;
 
-    let mut max_depth = 0;
-    for p in &prefixes {
-      max_depth = max(max_depth, p.val.len() as u32);
-    }
-    validate_prefix_tree(&prefixes, max_depth)?;
+    validate_prefix_tree(&prefixes)?;
 
-    let n_pref = 1_usize << max_depth;
-    let mut prefix_map = Vec::with_capacity(n_pref);
-    let mut prefix_len_map = Vec::with_capacity(n_pref);
-    for _ in 0..n_pref {
-      prefix_map.push(PrefixDecompressionInfo::new());
-      prefix_len_map.push(u32::MAX);
-    }
-    for p in &prefixes {
-      let i = bits::bits_to_usize_truncated(&p.val, max_depth);
-      prefix_map[i] = p.into();
-      prefix_len_map[i] = p.val.len() as u32;
-    }
-
-    let is_single_prefix = prefixes.len() == 1;
     Ok(ChunkDecompressor {
+      huffman_table: HuffmanTable::from(prefixes.clone()),
       prefixes,
-      prefix_map,
-      prefix_len_map,
-      max_depth,
       n,
-      is_single_prefix,
       compressed_body_size: metadata.compressed_body_size,
     })
   }
 
-  fn next_prefix(&self, reader: &mut BitReader) -> PrefixDecompressionInfo<T::Unsigned> {
-    if self.is_single_prefix {
-      self.prefix_map[0]
-    } else {
-      let mut prefix_idx = 0;
-      for prefix_len in 1..self.max_depth + 1 {
-        if reader.read_one() {
-          prefix_idx |= 1 << (self.max_depth - prefix_len);
-        }
-        if self.prefix_len_map[prefix_idx] == prefix_len {
-          return self.prefix_map[prefix_idx];
-        }
-      }
-      unreachable!("prefixes are corrupt yet not caught before decoding numbers");
-    }
-  }
-
-  pub fn decompress_chunk(&self, reader: &mut BitReader) -> QCompressResult<Vec<T>> {
-    let (start_byte_idx, _) = reader.inds();
+  // After much debugging a performance degradation from error handling changes,
+  // it turned out this function's logic ran slower when any heap allocations
+  // were done in the same scope. I don't understand why, but telling it not
+  // to inline fixed the performance issue.
+  // https://stackoverflow.com/questions/70911460/why-does-an-unrelated-heap-allocation-in-the-same-rust-scope-hurt-performance
+  #[inline(never)]
+  fn decompress_chunk_nums(&self, reader: &mut BitReader) -> Vec<T> {
     let n = self.n;
     let mut res = Vec::with_capacity(n);
     let mut i = 0;
     while i < n {
-      let p = self.next_prefix(reader);
+      let p = self.huffman_table.search_with_reader(reader);
 
       let reps = match p.run_len_jumpstart {
         None => {
@@ -144,17 +120,40 @@ impl<T> ChunkDecompressor<T> where T: NumberLike {
       }
       i += reps;
     }
+    res
+  }
 
-    let (end_byte_idx, _) = reader.inds();
-    let real_compressed_body_size = end_byte_idx - start_byte_idx;
-    if self.compressed_body_size != real_compressed_body_size {
-      return Err(QCompressError::CompressedBodySize {
-        expected: self.compressed_body_size,
-        actual: real_compressed_body_size,
-      });
+  fn validate_sufficient_data(&self, reader: &BitReader) -> QCompressResult<()> {
+    let start_byte_idx = reader.aligned_byte_ind()?;
+    let remaining_bytes = reader.size() - start_byte_idx;
+    if remaining_bytes < self.compressed_body_size {
+      Err(QCompressError::invalid_argument(format!(
+        "bit reader has only {} bytes remaining but compressed body size is {}",
+        remaining_bytes,
+        self.compressed_body_size,
+      )))
+    } else {
+      Ok(())
     }
+  }
+
+  pub fn decompress_chunk(&self, reader: &mut BitReader) -> QCompressResult<Vec<T>> {
+    self.validate_sufficient_data(reader)?;
+
+    let start_byte_idx = reader.aligned_byte_ind()?;
+    let res = self.decompress_chunk_nums(reader);
 
     reader.drain_byte();
+    let end_byte_idx = reader.aligned_byte_ind()?;
+    let real_compressed_body_size = end_byte_idx - start_byte_idx;
+    if self.compressed_body_size != real_compressed_body_size {
+      return Err(QCompressError::corruption(format!(
+        "expected the compressed body to contain {} bytes but it contained {}",
+        self.compressed_body_size,
+        real_compressed_body_size,
+      )));
+    }
+
     Ok(res)
   }
 }
@@ -167,8 +166,8 @@ impl<T> Debug for ChunkDecompressor<T> where T: NumberLike {
 
 #[derive(Clone, Debug, Default)]
 pub struct Decompressor<T> where T: NumberLike {
-  pub config: DecompressorConfig,
-  pub phantom: PhantomData<T>,
+  config: DecompressorConfig,
+  phantom: PhantomData<T>,
 }
 
 impl<T> Decompressor<T> where T: NumberLike {
@@ -180,59 +179,43 @@ impl<T> Decompressor<T> where T: NumberLike {
   }
 
   pub fn header(&self, reader: &mut BitReader) -> QCompressResult<Flags> {
-    let bytes = reader.read_bytes(MAGIC_HEADER.len())?;
+    let bytes = reader.read_aligned_bytes(MAGIC_HEADER.len())?;
     if bytes != MAGIC_HEADER {
-      return Err(QCompressError::MagicHeaderError {
-        header: bytes.to_vec()
-      });
+      return Err(QCompressError::corruption(format!(
+        "magic header does not match {:?}; instead found {:?}",
+        MAGIC_HEADER,
+        bytes,
+      )));
     }
-    let bytes = reader.read_bytes(1)?;
+    let bytes = reader.read_aligned_bytes(1)?;
     let byte = bytes[0];
     if byte != T::HEADER_BYTE {
-      return Err(QCompressError::HeaderDtypeError {
-        header_byte: byte,
-        decompressor_byte: T::HEADER_BYTE,
-      });
+      return Err(QCompressError::corruption(format!(
+        "data type byte does not match {:?}; instead found {:?}",
+        T::HEADER_BYTE,
+        byte,
+      )));
     }
 
     Flags::parse_from(reader)
   }
 
-  pub fn chunk_metadata(&self, reader: &mut BitReader, _flags: &Flags) -> QCompressResult<Option<ChunkMetadata<T>>> {
-    let magic_byte = reader.read_bytes(1)?[0];
+  pub fn chunk_metadata(&self, reader: &mut BitReader, flags: &Flags) -> QCompressResult<Option<ChunkMetadata<T>>> {
+    let magic_byte = reader.read_aligned_bytes(1)?[0];
     if magic_byte == MAGIC_TERMINATION_BYTE {
       return Ok(None);
     } else if magic_byte != MAGIC_CHUNK_BYTE {
-      return Err(QCompressError::MagicChunkByteError { byte: magic_byte });
+      return Err(QCompressError::corruption(format!(
+        "invalid magic chunk byte: {}",
+        magic_byte
+      )));
     }
 
     // otherwise there is indeed another chunk
-    let n = reader.read_usize(BITS_TO_ENCODE_N_ENTRIES as usize);
-    let compressed_body_size = reader.read_usize(BITS_TO_ENCODE_COMPRESSED_BODY_SIZE as usize);
-    let n_pref = reader.read_usize(MAX_MAX_DEPTH as usize);
-    let mut prefixes = Vec::with_capacity(n_pref);
-    for _ in 0..n_pref {
-      let count = reader.read_usize(BITS_TO_ENCODE_N_ENTRIES as usize);
-      let lower_bits = reader.read(T::PHYSICAL_BITS);
-      let lower = T::from_bytes(bits::bits_to_bytes(lower_bits));
-      let upper_bits = reader.read(T::PHYSICAL_BITS);
-      let upper = T::from_bytes(bits::bits_to_bytes(upper_bits));
-      let code_len = reader.read_usize(BITS_TO_ENCODE_PREFIX_LEN as usize);
-      let val = reader.read(code_len);
-      let jumpstart = if reader.read_one() {
-        Some(reader.read_usize(BITS_TO_ENCODE_JUMPSTART as usize))
-      } else {
-        None
-      };
-      prefixes.push(Prefix::new(count, val, lower, upper, jumpstart));
-    }
+    let metadata = ChunkMetadata::parse_from(reader, flags);
     reader.drain_byte();
 
-    Ok(Some(ChunkMetadata {
-      n,
-      compressed_body_size,
-      prefixes,
-    }))
+    Ok(Some(metadata))
   }
 
   pub fn decompress_chunk_body(
@@ -294,9 +277,9 @@ impl<T> Decompressor<T> where T: NumberLike {
 
 #[cfg(test)]
 mod tests {
-  use crate::{Decompressor, BitReader, Flags, ChunkMetadata};
+  use crate::{BitReader, ChunkMetadata, Decompressor, Flags};
+  use crate::errors::ErrorKind;
   use crate::prefix::Prefix;
-  use crate::errors::QCompressError;
 
   #[test]
   fn test_corrupt_prefixes_error_not_panic() {
@@ -327,7 +310,7 @@ mod tests {
         bad_metadata,
         &Flags::default(),
       );
-      assert!(matches!(result, Err(QCompressError::HuffmanTreeError {bits: _b, issue: _i})));
+      assert!(matches!(result.unwrap_err().kind, ErrorKind::Corruption));
     }
   }
 }
