@@ -29,7 +29,8 @@ mod bits;
 mod chunk_metadata;
 mod constants;
 mod flags;
-mod huffman;
+mod huffman_decoding;
+mod huffman_encoding;
 mod prefix;
 mod utils;
 pub mod bit_reader;
@@ -160,9 +161,9 @@ mod tests {
     );
   }
 
-  fn assert_recovers<T: NumberLike>(vals: Vec<T>, max_depth: u32) {
+  fn assert_recovers<T: NumberLike>(vals: Vec<T>, compression_level: u32) {
     let compressor = Compressor::<T>::from_config(
-      CompressorConfig { max_depth, ..Default::default()},
+      CompressorConfig { compression_level, ..Default::default()},
     );
     let compressed = compressor.simple_compress(&vals).expect("compression error");
     let decompressor = Decompressor::<T>::default();
