@@ -9,7 +9,15 @@ impl NumberLike for u32 {
   const HEADER_BYTE: u8 = 4;
   const PHYSICAL_BITS: usize = 32;
 
-  type Diff = u32;
+  type Unsigned = u32;
+
+  fn to_unsigned(self) -> Self::Unsigned {
+    self
+  }
+
+  fn from_unsigned(off: Self::Unsigned) -> Self {
+    off
+  }
 
   fn num_eq(&self, other: &Self) -> bool {
     self.eq(other)
@@ -19,14 +27,6 @@ impl NumberLike for u32 {
     self.cmp(other)
   }
 
-  fn offset_diff(upper: u32, lower: u32) -> u32 {
-    upper - lower
-  }
-
-  fn add_offset(lower: u32, off: u32) -> u32 {
-    lower + off
-  }
-
   fn bytes_from(num: u32) -> Vec<u8> {
     num.to_be_bytes().to_vec()
   }
