@@ -9,7 +9,15 @@ impl NumberLike for u64 {
   const HEADER_BYTE: u8 = 2;
   const PHYSICAL_BITS: usize = 64;
 
-  type Diff = u64;
+  type Unsigned = u64;
+
+  fn to_unsigned(self) -> Self::Unsigned {
+    self
+  }
+
+  fn from_unsigned(off: Self::Unsigned) -> Self {
+    off
+  }
 
   fn num_eq(&self, other: &Self) -> bool {
     self.eq(other)
@@ -19,14 +27,6 @@ impl NumberLike for u64 {
     self.cmp(other)
   }
 
-  fn offset_diff(upper: u64, lower: u64) -> u64 {
-    upper - lower
-  }
-
-  fn add_offset(lower: u64, off: u64) -> u64 {
-    lower + off
-  }
-
   fn bytes_from(num: u64) -> Vec<u8> {
     num.to_be_bytes().to_vec()
   }
